//////////////////////////////////////////////////////////////////////////////// 
/// Pseudoglobe Screensaver
/// 
/// Copyright  2002  Sebastian Pipping <webmaster@hartwork.org>
/// 
/// -->  http://www.hartwork.org
/// 
/// This source code is released under the GNU General Public License (GPL).
/// See GPL.txt for details. Any non-GPL usage is strictly forbidden.
////////////////////////////////////////////////////////////////////////////////


#include "ctimer.h"

CTimer::CTimer()
{
	iFrames = 0;
	bTimerFound = QueryPerformanceFrequency( ( LARGE_INTEGER *)&iFrequency );
	bTimerRunning = FALSE;

	iAdjustFactor = 2;
	iAdjustCount = 6;
}

void CTimer::Start()
{
	if( bTimerFound && !bTimerRunning )
	{
		QueryPerformanceCounter( ( LARGE_INTEGER * )&iBeginTime );
		bTimerRunning = TRUE;
	}
}

void CTimer::Pause()
{
	if( bTimerFound && bTimerRunning )
	{
		QueryPerformanceCounter( ( LARGE_INTEGER * )&iEndTime );
		iElapsedBefore = iEndTime - iBeginTime;
		bTimerRunning = FALSE;
	}
}

void CTimer::Continue()
{
	if( bTimerFound && !bTimerRunning )
	{
		QueryPerformanceCounter( ( LARGE_INTEGER * )&iBeginTime );
		iBeginTime -= iElapsedBefore;
		bTimerRunning = TRUE;
	}
}

void CTimer::Stop()
{
	if( bTimerFound && bTimerRunning )
	{
		QueryPerformanceCounter( ( LARGE_INTEGER * )&iEndTime );
		bTimerRunning = FALSE;
	}
}

double CTimer::GetElapsed()
{
	if( bTimerFound )
	{
		if( bTimerRunning )
		{
			QueryPerformanceCounter( ( LARGE_INTEGER * )&iEndTime );
		}
		return ( iEndTime - iBeginTime ) / ( double )iFrequency;
	}
	else
	{
		return -1;
	}
}

void CTimer::Adjust()
{
	if( bTimerFound && ( iFrames < 30241 ) ) // <= 30240
	{
		// Adjust animation and rotation
		double dFactor = 100.0 * GetElapsed() / iFrames;
		*pAnimationStep = ( float )( fAnimationStep100 * dFactor );
		*pRotationStep = ( float )( fRotationStep100 * dFactor );

		// Increase counter
		iAdjustCount *= iAdjustFactor; // 6, 12, 36, 144, 720, 4320, 30240
		iAdjustFactor++;
	}
}

BOOL CTimer::AdjustFrame()
{
	return( bTimerFound && ( iFrames == iAdjustCount ) );
}
